/******* BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/

 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is the Favicon Extension (A.K.A. Custicon), which is 
 * this file and the contents of the Custicon XPI Package which it came in
 * (from http://zenit.senecac.on.ca/wiki/imgs/Custicon@andrew.raynier.xpi),
 *.as it was released on December 14, 2007.
 * 
 * The Initial Developer of the Original Code is Andrew Raynier.
 * Portions created by Andrew Raynier are Copyright (C) 2007-2008
 * the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *  Simon Jung <sljung@learn.senecac.on.ca> (provided the zip of.ico Icons)
 * 
 * Developers of similar extensions (which this one is based off of):
 *   Justin Dolske <dolske@mozilla.com> (Original Author of Chromatabs)
 *   John P. Baker <john.p.baker@bristol.ac.uk> (Author of hashcolouredtabs)
 * 
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 ******* END LICENSE BLOCK *******/

var custicon = {
  
  _Components           : Components,
//  _contentprefService   : null,
  _debug                : true,
  _faviconService       : null,
  _historyService       : null,
  _logService           : null,
  _ioservice            : null,
//  _observer             : null,
  _prefs                : null,
  
  
  /**
   *  log - (created using nsIConsoleService)
   *        A log for debug messages to the Javascript console.
   *        Note: javascript.options.showInConsole must be enabled.
   *
   *  @param message : The string that's sent to the log.
   *                   Usually an error message or exception needed for a debug.
   *
   */
  log: function(message) {
    //check if debug is on...
    if (!this._debug) {return; }
    this._logService.logStringMessage("Custicon: " + message);
  },
  
  onLoad: function(e) {
    // initialization code
    this.init(e);
    //gets labels within locale
    this.strings = document.getElementById("custicon-strings");
    document.getElementById("contentAreaContextMenu")
            .addEventListener("popupshowing", this.showContextMenu(e), false);  //for webPage right-click menu.
  },
  
  //initializes Custicon variables when this script is loaded.
  init : function(e) {
    this._logService = Components.classes['@mozilla.org/consoleservice;1'].getService();
    this._logService.QueryInterface(Components.interfaces.nsIConsoleService);
    
    custicon.log("initializing...");
    
    this._prefs = Components.classes["@mozilla.org/preferences-service;1"]
                    .getService(Components.interfaces.nsIPrefService)
                    .getBranch(null);
    if(this._prefs){
        try{
          this._prefs.QueryInterface(Components.interfaces.nsIPrefBranch2);
          this._prefs.addObserver("extensions.custicon.", this, true);  //might fail (try 'false'), still testing.
        } catch (e){ custicon.log("prefs init: " + e); }
    }
    
//    this._observer = Components.classes["@mozilla.org/observer-service;1"]
//                       .getService(Components.interfaces.nsIObserverService);
//    if(this._observer){
//      this._observer.addObserver(this, "em-action-requested", true);
//    }
  
    try{
      this._faviconService = Components.classes["@mozilla.org/browser/favicon-service;1"]
                               .getService(Components.interfaces.nsIFaviconService);
      this._historyService = Components.classes["@mozilla.org/browser/nav-history-service;1"]
                               .getService(Components.interfaces.nsINavHistoryService);
    } catch (e){
      //places not enabled or Browser is pre-Firefox 3.
      custicon.log("places init: " + e);
      this._faviconService = null;
    }
  },
  
  //called by onMenuLoadCommand, after file has been successfully chosen.
  loadFavicon : function(file, fURL){ //image/vnd.microsoft.icon
    this._ioservice = Components.classes["@mozilla.org/network/io-service;1"]
                          .getService(Components.interfaces.nsIIOService);
    var pageURI = this._ioservice.newURI(gBrowser.currentURI.spec, null, null);
//    var pageURI = this._ioservice.newURI(fURL, null, null);
    var faviconURI = this._ioservice.newURI(fURL, null, null);
    
    var fileinput = this._Components.classes["@mozilla.org/network/file-input-stream;1"]
               .createInstance(this._Components.interfaces.nsIFileInputStream);
    var bininput = this._Components.classes['@mozilla.org/binaryinputstream;1']
               .createInstance(this._Components.interfaces.nsIBinaryInputStream);
    
    var mimeservice = this._Components.classes['@mozilla.org/mime;1']
               .createInstance(this._Components.interfaces.nsIMIMEService);
    
    var type = mimeservice.getTypeFromFile(file);
    
    if(!type) type = "application/octet-stream";  //backup mime type incase [get mime file type] doesn't work.
    
    fileinput.init(file, 0x01, 0,5);
    bininput.setInputStream(fileinput);
    var fileCounts = fileinput.available();
    var fileContents = bininput.readByteArray(fileCounts);
    bininput.close();
    fileinput.close();
    
    try{
      this._faviconService.setFaviconData(faviconURI, fileContents, fileCounts, type, 0);
      this._faviconService.setAndLoadFaviconForPage(pageURI, faviconURI, false);
      this._faviconService.setFaviconUrlForPage(pageURI, faviconURI);
      //custicon.changeTabIcon(pageURI, faviconURI);
    } catch (e){
      //this command itself causes an exception.
      //this._faviconService.setFaviconUrlForPage(gBrowser.currentURI.spec, faviconURI);
      custicon.log("Loading Favicon: " + e);
    }
    
    custicon.changeTabIcon(fURL, faviconURI);
    custicon.onMenuItemCommand();
    custicon.log("Favicon loaded!? = " + file + ", @= " + pageURI);
  },
  
  QueryInterface : function(aIID){
    if( aIID.equals(Components.interfaces.nsIObserver) ||
        aIID.equals(Components.interfaces.nsIContentPrefObserver) ||
        aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
        aIID.equals(Components.interfaces.nsISupports)) {
      return this;
    }
    throw Components.results.NS_ERROR_NO_INTERFACE;
  },
  
  showContextMenu: function(e) {
    // show or hide the menuitem based on what the context menu is on
    // see http://kb.mozillazine.org/Adding_items_to_menus
    var show = document.getElementById("context-custicon");
    show.hidden = gContextMenu.onImage;
  },
  
  onMenuItemCommand: function(e) {
    var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                                  .getService(Components.interfaces.nsIPromptService);
    promptService.alert(window, this.strings.getString("helloMessageTitle"),
                                this.strings.getString("helloMessage"));
  },
  
  onMenuLoadCommand: function(e) {
    var filePicker = Components.classes["@mozilla.org/filepicker;1"]
                               .createInstance(Components.interfaces.nsIFilePicker);
    filePicker.init(window, "Load Favicon", filePicker.modeOpen);
    filePicker.appendFilters(filePicker.filterAll | filePicker.filterImages); //load filter sets
    filePicker.appendFilter("Icon Files (*.ico)","*.ico");
    //get profile directory to show as the default displayed folder on dialog box.
    filePicker.displayDirectory = Components.classes["@mozilla.org/file/directory_service;1"]
                     .getService(Components.interfaces.nsIProperties)
                     .get("ProfD", Components.interfaces.nsIFile);
    // calls and shows the Open dialog box.
    var returnValue = filePicker.show();
    
    //checks for an appropriate dialog box response. (i.e. if the 'Open' was cancelled or not.)
    if (returnValue == filePicker.returnOK) {
      var file = filePicker.file;
      
      // Get the path as string. Note that you usually won't need to work with the string paths.
      var fURL = filePicker.fileURL.spec;
      
      // work with returned nsILocalFile...
      custicon.loadFavicon(file, fURL);
      
      //custicon.changeTabIcon(file, fURL);
    }
  },
  
  changeTabIcon: function(fURL, faviconURI){
    //Jan '08
    var _doc = gBrowser.contentDocument;
    var _tab = gBrowser.mTabs[gBrowser.getBrowserIndexForDocument(_doc)];
       //or tab = event.originalTarget; ?? or even tab = event.target; ?
    var imgsrc1 = _tab.getAttribute("image");
    try{
        gBrowser.setIcon(_tab, fURL);
    } catch (e){
        custicon.log("setIcon: " + e);
    }
    
    //pre 2008 
    /*
    var tab = getBrowser().mContextTab;
    var tabIcon = getAnonymousElementByAttribute(tab, "class", "tab-icon-image");
    if (tab.localName != "tab")
      tab = getBrowser().mCurrentTab;
    
    tab.tabIcon.setAttribute("src", faviconURI.spec);
    this.onMenuItemCommand(e); 
    return true;*/
  }

};

//window.addEventListener("unload", function(e) { custicon.onUnload(e); }, false);
window.addEventListener("load", function(e) { custicon.onLoad(e); }, false);
